/*
	bone_ppp_neg.h
	
	ppp options negotiator - a 'sub-class' of bone_ppp_handler which
	implements the standard ppp negotiation state-machine.
	
	includes utilities for managing lists of ppp options

	Copyright 2001, Be Incorporated, All Rights Reserved.
*/

#ifndef H_BONE_PPP_NEG
#define H_BONE_PPP_NEG

#include <module.h>
#include <bone_util.h>
#include <bone_ppp_session.h>

// Configuration Protocol Type Codes Mask
enum
{
	PPPCM_CONFIG_REQ = 	0x00000002,
	PPPCM_CONFIG_ACK = 	0x00000004,
	PPPCM_CONFIG_NAK = 	0x00000008,
	PPPCM_CONFIG_REJ = 	0x00000010,
	PPPCM_TERM_REQ = 	0x00000020,
	PPPCM_TERM_ACK = 	0x00000040,
	PPPCM_CODE_REJ = 	0x00000080,
	PPPCM_PROTO_REJ = 	0x00000100,
	PPPCM_ECHO_REQ = 	0x00000200,
	PPPCM_ECHO_REP = 	0x00000400,
	PPPCM_DISCARD_REQ = 0x00000800,
	PPPCM_IDENT = 		0x00001000,
	PPPCM_TIME_REMAIN = 0x00002000,
	PPPCM_RESET_REQ = 	0x00004000,
	PPPCM_VENDOR_EXT = 	0x00000001,
};

// ***
// Configuration Proto Option Structs
// ***

// CP Header
typedef struct cp_header
{
	uint8		code;
	uint8		id;
	uint16		length;
} cp_header_t;

// CP option node; used in CP options list
typedef struct cp_option
{
	struct cp_option *next;
	struct cp_option **prev;
	
	uint8		type;
	uint8		len;		// Only includes data - two byte header _not_ included
	char		data[0];
} cp_option_t;

// CP options list
typedef struct cp_options
{
	int32			count;
	int32			total_len;
	cp_option_t		*head;
} cp_options_t;

// ppp_negotiator is a 'sub-class' of ppp_handler which implements the standard
// ppp negotiation state-machine. It implements all of the 'base-class' functions.
typedef struct bone_ppp_negotiator {
	// Fillout handler.config_proto, handler.data_protos, and handler.level
	bone_ppp_handler_t		handler;
	
	uint32		code_mask;
	void		*cookie;
	
	status_t	(*send_data)(struct bone_ppp_negotiator *you, bone_data_t *data);
	status_t	(*receive_nak)(struct bone_ppp_negotiator *you, cp_options_t *req, cp_options_t *nak);
	status_t	(*receive_rej)(struct bone_ppp_negotiator *you, cp_options_t *req, cp_options_t *rej);
	status_t	(*receive_conf_req)(struct bone_ppp_negotiator *you, struct cp_options *req, struct cp_options *nak, struct cp_options *rej);
	status_t	(*receive_data)(struct bone_ppp_negotiator *you, struct bone_data *data, uint16 ppp_proto);
	
	void		(*started)(struct bone_ppp_negotiator *you, struct cp_options *req);
	void		(*finished)(struct bone_ppp_negotiator *you);
	void		(*up)(struct bone_ppp_negotiator *you, struct cp_options *req, struct cp_options *ack);
	void		(*down)(struct bone_ppp_negotiator *you);
} bone_ppp_negotiator_t;

typedef struct bone_ppp_neg_info {

	struct module_info module;
	
	status_t	(*init_negotiator)(struct bone_ppp_negotiator *neg);
	status_t	(*uninit_negotiator)(struct bone_ppp_negotiator *neg);
	
	/*
	 * Configuration Option Utils
	 */
	
	/* Create and delete PPP options lists */
	cp_options_t *		(*new_options)(void);
	cp_options_t *		(*duplicate_options)(cp_options_t *options);
	void				(*clear_options)(cp_options_t *options);
	void 				(*delete_options)(cp_options_t *options);
	
	/* Add a new option of length 'len' to the options list; returns pointer to 'data' portion of option */
	char *				(*add_option)(cp_options_t *options, uint8 type, uint16 len);
	
	/* Remove an option 'type' from list */
	status_t 			(*delete_option)(cp_options_t *options, uint8 type);
	
	/* Locate option of specified 'type' in the list and return a pointer */
	cp_option_t *		(*find_option)(cp_options_t *options, uint8 type);
	
	/* Add a copy of 'option' to the list 'options' */
	status_t 			(*copy_option)(cp_options_t *options, const cp_option_t *option);
} bone_ppp_neg_info_t;

#define BONE_NEG_MODULE_NAME "negotiator"

#endif
